@@ -51,8 +51,10 @@ using namespace std;
 //' @field hasOTHER Test if current variant has a OTHER or not
 //' @field hasOVERLAP Test if current variant has a OVERLAP or not
 //' @field nsamples Return the number of samples
-//' @field string Return the raw string of current variant
+//' @field samples Return a vector of samples id
 //' @field header Return the raw string of the vcf header
+//' @field string Return the raw string of current variant including newline
+//' @field line Return the raw string of current variant without newline
 //' @field output Init an output object for streaming out the variants to another vcf
 //' @field write Streaming out current variant the output vcf
 //' @field close Close the connection to the output vcf
@@ -90,6 +92,8 @@ using namespace std;
 //' \item Parameter: s - A string for the tag value}
 //' @field rmInfoTag Remove the given tag from the INFO of current variant
 //' \itemize{\item Parameter: s - A string for the tag name}
+//' @field rmFormatTag Remove the given tag from the FORMAT of current variant
+//' \itemize{\item Parameter: s - A string for the tag name}
 //' @field setVariant Modify current variant by adding a vcf line
 //' \itemize{\item Parameter: s - A string for one line in the VCF}
 //' @field addINFO Add a INFO in the header of the vcf
@@ -243,13 +247,20 @@ class vcfreader {
         auto v = genotypes(false);
         return v.size() / nsamples();
     }
-    inline std::string string() const { return var.asString(); }
     inline std::string header() const { return br.header.asString(); }
+    inline std::vector<std::string> samples() const { return br.header.getSamples(); }
+    inline std::string string() const { return var.asString(); }
+    inline std::string line() {
+        std::string s = var.asString();
+        s.pop_back();
+        return s;
+    }
 
     // WRITE
     inline void output(const std::string& vcffile) {
         bw.open(vcffile);
         bw.initalHeader(br.header);
+        writable = true;
     }
     inline void write() { bw.writeRecord(var); }
     inline void close() { bw.close(); }
@@ -258,22 +269,30 @@ class vcfreader {
     inline void setID(std::string s) { var.setID(s.c_str()); }
     inline void setPOS(int pos) { var.setPOS(pos); }
     inline void setRefAlt(const std::string& s) { var.setRefAlt(s.c_str()); }
-    inline void setInfoInt(std::string tag, int v) { var.setINFO(tag, v); }
-    inline void setInfoFloat(std::string tag, double v) { var.setINFO(tag, v); }
-    inline void setInfoStr(std::string tag, const std::string& s) { var.setINFO(tag, s); }
-    inline void setFormatInt(std::string tag, const vector<int>& v) { var.setFORMAT(tag, v); }
-    inline void setFormatFloat(std::string tag, const vector<double>& v) {
-        vector<float> f(v.begin(), v.end());
-        var.setFORMAT(tag, f);
+    inline bool setInfoInt(std::string tag, int v) { return var.setINFO(tag, v); }
+    inline bool setInfoFloat(std::string tag, double v) { return var.setINFO(tag, v); }
+    inline bool setInfoStr(std::string tag, const std::string& s) { return var.setINFO(tag, s); }
+    inline bool setFormatInt(std::string tag, const vector<int>& v) {
+        return var.setFORMAT(tag, v);
     }
-    inline void setFormatStr(std::string tag, const std::string& s) { var.setINFO(tag, s); }
-    inline void setGenotypes(const vector<int>& v) {
+    inline bool setFormatFloat(std::string tag, const vector<double>& v) {
+        vector<float> f(v.begin(), v.end());
+        return var.setFORMAT(tag, f);
+    }
+    inline bool setFormatStr(std::string tag, const std::string& s) {
+        if (s.length() % nsamples() != 0) {
+            Rcpp::Rcout << "the length of s must be divisable by nsamples()";
+            return false;
+        }
+        return var.setFORMAT(tag, s);
+    }
+    inline bool setGenotypes(const vector<int>& v) {
         if ((int)v.size() != nsamples() * ploidy()) {
             Rcpp::Rcout << "nsamples: " << nsamples() << ", ploidy: " << ploidy() << "\n";
-            throw std::runtime_error(
-                "the size of genotype vector is not equal to nsamples * ploidy");
+            Rcpp::Rcout << "the size of genotype vector is not equal to nsamples * ploidy\n";
+            return false;
         }
-        var.setGenotypes(v);
+        return var.setGenotypes(v);
     }
     inline void setPhasing(const vector<int>& v) {
         vector<char> c(v.begin(), v.end());
@@ -281,17 +300,24 @@ class vcfreader {
     }
 
     inline void rmInfoTag(std::string s) { var.removeINFO(s); }
-    inline void setVariant(const std::string& s) { var.addLineFromString(s); }
+    inline void rmFormatTag(std::string s) { var.removeFORMAT(s); }
     inline void addINFO(const std::string& id, const std::string& number, const std::string& type,
                         const std::string& desc) {
-        bw.header.addINFO(id, number, type, desc);
+        if (writable)
+            bw.header.addINFO(id, number, type, desc);
+        else
+            Rcpp::Rcout << "please call the `output(filename)` function first\n";
     }
     inline void addFORMAT(const std::string& id, const std::string& number, const std::string& type,
                           const std::string& desc) {
-        bw.header.addFORMAT(id, number, type, desc);
+        if (writable)
+            bw.header.addFORMAT(id, number, type, desc);
+        else
+            Rcpp::Rcout << "please call the `output(filename)` function first\n";
     }
 
    private:
+    bool writable = false;
     vcfpp::BcfReader br;
     vcfpp::BcfRecord var;
     vcfpp::BcfWriter bw;
@@ -342,8 +368,10 @@ RCPP_MODULE(vcfreader) {
         .method("hasOTHER", &vcfreader::hasOTHER)
         .method("hasOVERLAP", &vcfreader::hasOVERLAP)
         .method("nsamples", &vcfreader::nsamples)
+        .method("samples", &vcfreader::samples)
         .method("header", &vcfreader::header)
         .method("string", &vcfreader::string)
+        .method("line", &vcfreader::line)
         .method("output", &vcfreader::output)
         .method("write", &vcfreader::write)
         .method("close", &vcfreader::close)
@@ -359,8 +387,8 @@ RCPP_MODULE(vcfreader) {
         .method("setFormatInt", &vcfreader::setFormatInt)
         .method("setFormatFloat", &vcfreader::setFormatFloat)
         .method("setFormatStr", &vcfreader::setFormatStr)
-        .method("setVariant", &vcfreader::setVariant)
         .method("addINFO", &vcfreader::addINFO)
         .method("addFORMAT", &vcfreader::addFORMAT)
-        .method("rmInfoTag", &vcfreader::rmInfoTag);
+        .method("rmInfoTag", &vcfreader::rmInfoTag)
+        .method("rmFormatTag", &vcfreader::rmFormatTag);
 }
